pub mod meili;
pub mod tantivy;

/// Trait representing a search index backend;
/// either Tantivy (embedded) or Meilisearch (via HTTP API).
pub trait Backend: Send + Sync {
    fn add_document(&mut self, document: BackendIndependentDocument) -> anyhow::Result<()>;

    fn flush(&mut self) -> anyhow::Result<()>;

    fn query(&self, query: String) -> anyhow::Result<Vec<SearchDocument>>;
}

/// A backend-independent document struct.
#[derive(Clone, Debug)]
pub struct BackendIndependentDocument {
    pub title: String,
    pub article_body: String,
    pub nonarticle_body: String,
    pub tags: Vec<String>,
    pub url: String,
    pub favicon_url_hash: u64,
}

/// A backend-independent document struct.
#[derive(Clone, Debug)]
pub struct SearchDocument {
    pub score: f32,
    pub title: String,
    pub excerpt: String,
    pub tags: Vec<String>,
    pub url: String,
    pub favicon_url_hash: u64,
}
