use anyhow::Context;
use clap::Parser;
use colour::{dark_grey_ln, yellow_ln};
use env_logger::Env;

use quickpeep_indexer::config::IndexerConfig;

use std::path::PathBuf;

/// Seeds a raker's queue with URLs
#[derive(Clone, Debug, Parser)]
pub struct Opts {
    #[clap(long = "config")]
    config: Option<PathBuf>,

    query: String,
}

pub fn main() -> anyhow::Result<()> {
    env_logger::Builder::from_env(
        Env::default().default_filter_or("info,tantivy=warn,qp_index_search=debug"),
    )
    .init();

    let opts: Opts = Opts::parse();

    let config_path = opts
        .config
        .unwrap_or_else(|| PathBuf::from("quickpeep.ron"));
    let config = IndexerConfig::load(&config_path).context("Failed to load config")?;

    let indexer_backend = config.open_indexer_backend()?;

    let results = indexer_backend.query(opts.query)?;

    for result in results {
        yellow_ln!("{}", result.title);
        dark_grey_ln!("\t[{:.3}] {}", result.score, result.url);
        // TODO Excerpts
        println!();
    }

    Ok(())
}
