-- All timestamps are in sec since 2023-01-01.

CREATE TYPE rakeintent AS ENUM ('Any', 'Page', 'Feed', 'SiteMap', 'Icon');

CREATE TABLE domains (
    domain_id SERIAL PRIMARY KEY NOT NULL,
    name TEXT UNIQUE NOT NULL,
    domain_record JSONB NOT NULL
);

CREATE TABLE urls (
    url_id SERIAL PRIMARY KEY NOT NULL,
    domain_id INTEGER NOT NULL REFERENCES domains(domain_id),
    url TEXT NOT NULL,
    intent rakeintent NOT NULL,
    visited_at_ts INTEGER,

    UNIQUE (domain_id, url)
);

CREATE TABLE url_queue (
    url_id INTEGER PRIMARY KEY NOT NULL REFERENCES urls(url_id),
    rake_after_ts INTEGER NOT NULL
);
-- Used for finding things to rake.
CREATE INDEX url_queue_rake_after_ts_idx ON url_queue(rake_after_ts);


CREATE TABLE active_domain_raffle (
    domain_id INTEGER PRIMARY KEY NOT NULL REFERENCES domains(domain_id),
    raffle INTEGER UNIQUE NOT NULL
);


CREATE TABLE domain_backoffs (
    domain_id INTEGER PRIMARY KEY NOT NULL REFERENCES domains(domain_id),
    backoff_until_ts INTEGER NOT NULL,
    backoff_sec INTEGER NOT NULL,
    reason TEXT NOT NULL
);
-- Used for finding things to rake.
CREATE INDEX domain_backoffs_backoff_until_ts_idx ON domain_backoffs(backoff_until_ts);
