use crate::storage::records::DomainRecord;
use crate::storage::RakerTxn;
use itertools::Itertools;
use sqlx::query_as;
use sqlx::types::Json;

/// Re-enqueues domains
/// Ideally should be applied after importing seeds and weeds on an existing database.
pub async fn reapply_seeds_and_weeds_to_on_hold_urls(
    mut txn: RakerTxn<'_, '_, true>,
) -> anyhow::Result<()> {
    struct DomainState {
        pub domain_id: i32,
        pub domain_record: Json<DomainRecord>,
    }

    let reinstatable_domains: Vec<DomainState> = query_as!(DomainState, r#"
        SELECT DISTINCT u.domain_id, domain_record AS "domain_record: Json<DomainRecord>" FROM url_queue
        JOIN urls u USING (url_id)
        JOIN domains USING (domain_id)
        LEFT JOIN domain_backoffs db ON u.domain_id = db.domain_id
        WHERE db.domain_id IS NULL
    "#).fetch_all(&mut *txn.txn)
        .await?;

    for domain_state in reinstatable_domains {
        let any_seeds = domain_state
            .domain_record
            .rakeable_path_prefixes
            .values()
            .contains(&true);
        if !any_seeds {
            continue;
        }

        // This domain has *some* seeds, let's just reinstate it.
        txn.insert_active_domain_id_with_new_raffle_ticket(domain_state.domain_id)
            .await?;
    }
    Ok(())
}
