use crate::raking::RakeIntent;
use reqwest::Url;
use serde::{Deserialize, Serialize};
use std::collections::BTreeMap;

#[derive(Clone, Debug, Deserialize, Serialize)]
pub struct QueueUrlRecord {
    pub intent: RakeIntent, // TODO CONSIDER
}

#[derive(Clone, Debug, Serialize, Deserialize, Default)]
pub struct DomainRecord {
    pub rakeable_path_prefixes: BTreeMap<String, bool>,
}

impl DomainRecord {
    /// Returns whether the URL is rakeable.
    ///
    /// Preconditions: it has been checked that the record applies to the domain
    pub fn is_url_rakeable(&self, url: &Url) -> Option<bool> {
        let mut final_result = None;
        // TODO This could be made more efficient.
        for (prefix, &rakeable) in self.rakeable_path_prefixes.iter() {
            if url.path().starts_with(prefix) {
                final_result = Some(rakeable);
            }
            if prefix.as_str() > url.path() {
                // e.g. /dog > /cat/xyz
                // This means we've missed all chances to see our prefix,
                // so we break here (efficiency).
                break;
            }
        }
        final_result
    }
}
