use crate::raking::{decode_robots_txt, rake_feed, rake_sitemap, RakeIntent, UrlRaked};
use reqwest::Url;

#[test]
pub fn test_sitemap() {
    let sm = rake_sitemap(
        br#"
    <urlset><url><loc>https://example.org/index.html</loc></url></urlset>
    "#,
    )
    .unwrap();
    assert_eq!(
        sm,
        vec![UrlRaked {
            url: Url::parse("https://example.org/index.html").unwrap(),
            last_changed: None,
            intent: RakeIntent::Page
        },]
    );

    let sm = rake_sitemap(
        br#"
    <sitemapindex><sitemap><loc>https://example.org/sitemap.xml</loc></sitemap></sitemapindex>
    "#,
    )
    .unwrap();
    assert_eq!(
        sm,
        vec![UrlRaked {
            url: Url::parse("https://example.org/sitemap.xml").unwrap(),
            last_changed: None,
            intent: RakeIntent::SiteMap
        }]
    );
}

#[test]
pub fn test_feed() {
    let feed = rake_feed(
        br#"
    <feed>
        <entry>
            <link href="https://example.org/index.html" />
        </entry>
    </feed>
    "#,
        &Url::parse("https://example.org/atom.xml").unwrap(),
    )
    .unwrap();

    assert_eq!(
        feed,
        vec![UrlRaked {
            url: Url::parse("https://example.org/index.html").unwrap(),
            last_changed: None,
            intent: RakeIntent::Page
        },]
    );
}

#[tokio::test]
pub async fn test_robots_txt() {
    let rtxt = decode_robots_txt(
        br#"
User-Agent: *
Disallow: /bad
Allow: /bad/abc

SiteMap: https://example.org/sitemap.xml
    "#,
    )
    .await
    .unwrap()
    .unwrap();
    assert_eq!(
        rtxt.sitemaps,
        vec![UrlRaked {
            url: Url::parse("https://example.org/sitemap.xml").unwrap(),
            last_changed: None,
            intent: RakeIntent::SiteMap
        }]
    );

    assert!(!rtxt.rules.allow("/bad"));
    assert!(!rtxt.rules.allow("/bad/def"));
    assert!(rtxt.rules.allow("/bad/abc"));
    assert!(rtxt.rules.allow("/good"));
}
