use crate::{parse_seeds, Seed, SeedBlock};

#[test]
fn test_simple_seeds() {
    let parsed = parse_seeds(
        r#"
Blog, Example:
https://example.org
    "#,
    )
    .unwrap();

    assert_eq!(
        parsed,
        vec![SeedBlock {
            tags: vec!["Blog".into(), "Example".into()],
            seeds: vec![Seed {
                url: "https://example.org".to_string(),
                extra_tags: vec![]
            }]
        }]
    );
}

#[test]
fn test_complicated_seeds() {
    let parsed = parse_seeds(
        r#"
Blog, Example:
https://example.org
# This demonstrates fancy extra tags
https://example.com [Commercial]

# This demonstrates another seed block
Software:
https://www2.example.org
    "#,
    )
    .unwrap();

    assert_eq!(
        parsed,
        vec![
            SeedBlock {
                tags: vec!["Blog".into(), "Example".into()],
                seeds: vec![
                    Seed {
                        url: "https://example.org".to_string(),
                        extra_tags: vec![]
                    },
                    Seed {
                        url: "https://example.com".to_string(),
                        extra_tags: vec!["Commercial".into()]
                    }
                ]
            },
            SeedBlock {
                tags: vec!["Software".into()],
                seeds: vec![Seed {
                    url: "https://www2.example.org".to_string(),
                    extra_tags: vec![]
                }]
            }
        ]
    );
}
